@@ -15,6 +15,7 @@ import java.security.cert.CertificateParsingException;
 import java.security.cert.CollectionCertStoreParameters;
 import java.security.cert.PKIXParameters;
 import java.security.cert.TrustAnchor;
+import java.security.cert.X509CertSelector;
 import java.security.cert.X509Certificate;
 import java.util.ArrayList;
 import java.util.Arrays;
@@ -32,10 +33,10 @@ import org.bouncycastle.asn1.x500.RDN;
 import org.bouncycastle.asn1.x500.X500Name;
 import org.bouncycastle.asn1.x500.style.BCStyle;
 import org.bouncycastle.i18n.ErrorBundle;
+import org.bouncycastle.jcajce.PKIXExtendedParameters;
 import org.bouncycastle.jce.provider.BouncyCastleProvider;
 import org.bouncycastle.x509.CertPathReviewerException;
 import org.bouncycastle.x509.PKIXCertPathReviewer;
-import org.bouncycastle.x509.X509CertStoreSelector;
 
 import eu.emi.security.authn.x509.RevocationParameters;
 import eu.emi.security.authn.x509.ValidationError;
@@ -99,6 +100,12 @@ public class BCCertPathValidator
 	 * </ul>
 	 * 
 	 * @param toCheck chain to check
+	 * @param proxySupport proxy support
+	 * @param trustAnchors trust anchors
+	 * @param crlStore crl store
+	 * @param revocationParams revocation params
+	 * @param observersHandler observers handler
+	 * @return validation result
 	 * @throws CertificateException if some of the certificates in the chain can not 
 	 * be parsed
 	 */
@@ -126,7 +133,7 @@ public class BCCertPathValidator
 		
 		if (!proxySupport || !ProxyUtils.isProxy(toCheck))
 		{
-			ExtPKIXParameters params = createPKIXParameters(toCheck, proxySupport, 
+			ExtPKIXParameters2 params = createPKIXParameters(toCheck, proxySupport, 
 					trustAnchors, crlStore, revocationParams, observersHandler);
 			List<X509Certificate> chain = checkNonProxyChain(toCheck, params, errors, unresolvedExtensions, 0, toCheck);
 			return new ValidationResult(errors.size() == 0, errors, unresolvedExtensions, chain);
@@ -147,7 +154,7 @@ public class BCCertPathValidator
 		for (int i=0; i<split+2; i++)
 			proxyChain[i] = toCheck[i];
 		
-		ExtPKIXParameters params = createPKIXParameters(baseChain, proxySupport, 
+		ExtPKIXParameters2 params = createPKIXParameters(baseChain, proxySupport, 
 				trustAnchors, crlStore, revocationParams, observersHandler);
 		List<X509Certificate> validatedChain = checkNonProxyChain(baseChain, params, errors, unresolvedExtensions, split+1, toCheck);
 			
@@ -158,7 +165,7 @@ public class BCCertPathValidator
 			trustForProxyChain = trustAnchors;
 		checkProxyChainWithBC(proxyChain, trustForProxyChain, errors, unresolvedExtensions);
 		
-		checkProxyChainMain(proxyChain, errors, unresolvedExtensions, params.getDate());
+		checkProxyChainMain(proxyChain, errors, unresolvedExtensions, params.getBaseParameters().getDate());
 		if (errors.size() == 0 && validatedChain != null)
 		{
 			for (int j=proxyChain.length-2; j>=0; j--)
@@ -167,22 +174,23 @@ public class BCCertPathValidator
 		return new ValidationResult(errors.size() == 0, errors, unresolvedExtensions, validatedChain);
 	}
 	
-	protected ExtPKIXParameters createPKIXParameters(X509Certificate[] toCheck, boolean proxySupport,
+	protected ExtPKIXParameters2 createPKIXParameters(X509Certificate[] toCheck, boolean proxySupport,
 			Set<TrustAnchor> trustAnchors, CertStore crlStore, 
 			RevocationParameters revocationParams, ObserversHandler observersHandler)
 	{
-		ExtPKIXParameters params;
-		X509CertStoreSelector endSelector = new X509CertStoreSelector();
+		X509CertSelector endSelector = new X509CertSelector();
 		endSelector.setCertificate(toCheck[0]);
+		PKIXParameters baseOfBase;
 		try
 		{
-			params = new ExtPKIXParameters(trustAnchors, endSelector, observersHandler);
+			baseOfBase = new PKIXParameters(trustAnchors);
 		} catch (InvalidAlgorithmParameterException e)
 		{
-			throw new RuntimeException("BUG, never should happen", e);
+			throw new IllegalStateException("Can't create PKIXParameters, shouldn't happen", e);
 		}
-		params.addCertStore(crlStore);
-		
+		baseOfBase.setTargetCertConstraints(endSelector);
+		baseOfBase.setDate(new Date());
+		baseOfBase.addCertStore(crlStore);
 		CertStore certStore;
 		try
 		{
@@ -194,11 +202,14 @@ public class BCCertPathValidator
 			throw new RuntimeException("Can't create an instance of a " +
 					"simple Collection certificate store, using the BC provider, BUG?", e1);
 		}
-		params.addCertStore(certStore);
-		params.setRevocationParams(revocationParams);
-		params.setProxySupport(proxySupport);
-		params.setDate(new Date());
-		return params;
+		baseOfBase.addCertStore(certStore);
+		
+		PKIXExtendedParameters.Builder baseBuilder = new PKIXExtendedParameters.Builder(baseOfBase);
+		ExtPKIXParameters2.Builder paramsBuilder = new ExtPKIXParameters2.Builder(
+				baseBuilder, baseOfBase, trustAnchors, observersHandler);
+		paramsBuilder.setRevocationParams(revocationParams);
+		paramsBuilder.setProxySupport(proxySupport);
+		return paramsBuilder.build();
 	}
 	
 	protected int getFirstProxy(X509Certificate[] toCheck)
@@ -260,15 +271,17 @@ public class BCCertPathValidator
 	 * Performs checking of the chain which has no proxies (or at least should not have proxies),
 	 * using {@link FixedBCPKIXCertPathReviewer}. In future, when BC implementation is fixed
 	 * it should use {@link PKIXCertPathReviewer} instead.  
-	 * @param baseChain
-	 * @param params
-	 * @param errors
-	 * @param unresolvedExtensions
+	 * @param baseChain base chain
+	 * @param params parameters
+	 * @param errors errors
+	 * @param unresolvedExtensions unresolved extensions
+	 * @param posDelta position delta
+	 * @param cc certificate chain
 	 * @return validated chain or null
-	 * @throws CertificateException
+	 * @throws CertificateException certificate exception
 	 */
 	protected List<X509Certificate> checkNonProxyChain(X509Certificate[] baseChain, 
-			ExtPKIXParameters params, List<ValidationError> errors, 
+			ExtPKIXParameters2 params, List<ValidationError> errors, 
 			Set<String> unresolvedExtensions, int posDelta, X509Certificate[] cc) 
 					throws CertificateException
 	{
@@ -277,7 +290,7 @@ public class BCCertPathValidator
 		List<ValidationError> buildPathErrors = null;
 		try
 		{
-			certPaths = builder.buildPath(params, baseChain[0], cc);
+			certPaths = builder.buildPath(params.getBaseBuildParameters(), baseChain[0], cc);
 		} catch (ValidationErrorException e1)
 		{
 			buildPathErrors = e1.getErrors();
@@ -297,12 +310,12 @@ public class BCCertPathValidator
 			} catch (CertPathReviewerException e)
 			{
 				//really shoudn't happen - we have checked the arguments
-				throw new RuntimeException("Can't init PKIXCertPathReviewer, bug?", e);
+				throw new IllegalStateException("Can't init PKIXCertPathReviewer, bug?", e);
 			}
 			if (buildPathErrors != null && baseReviewer.isValidCertPath())
 			{
 				//ups!!! bad! PKIXCertPAthReviewer validated while the path was not even build
-				throw new RuntimeException("PKIXCertPAthReviewer validated while the path was not even " +
+				throw new IllegalStateException("PKIXCertPAthReviewer validated while the path was not even " +
 					"build correctly. Build path error: " + buildPathErrors.get(0));
 			}
 			
@@ -334,7 +347,7 @@ public class BCCertPathValidator
 				unresolvedExtensions.addAll(getUnresolvedExtensionons(rawErrors));
 		} else
 		{
-			throw new RuntimeException("PKIXCertPAthReviewer BUG: validationErrors is null, " +
+			throw new IllegalStateException("PKIXCertPAthReviewer BUG: validationErrors is null, " +
 					"tested chain: " + CertificateUtils.format(baseChain, FormatMode.FULL));
 		}
 		return null;
@@ -345,10 +358,11 @@ public class BCCertPathValidator
 	 * EEC issuer is used as the only trust anchor. CRLs are ignored, proxy extension OIDs 
 	 * are marked as handled. The error resulting from the missing CA extension is
 	 * ignored as well as validity time errors. The latter are checked manually later on.
-	 * @param proxyChain
-	 * @param errors
-	 * @param unresolvedExtensions
-	 * @throws CertificateException
+	 * @param proxyChain proxy chain
+	 * @param trustAnchor trust anchor
+	 * @param errors errors
+	 * @param unresolvedExtensions unresolved extensions
+	 * @throws CertificateException certificate exception
 	 */
 	protected void checkProxyChainWithBC(X509Certificate[] proxyChain, 
 			Set<TrustAnchor> trustAnchor, 
@@ -380,10 +394,11 @@ public class BCCertPathValidator
 	 * Performs a validation loop of the proxy chain checking each pair in chain
 	 * for the rules not otherwise verified by the base check. Additionally chain length
 	 * restriction is verified.
-	 * @param proxyChain
-	 * @param errors
-	 * @param unresolvedExtensions
-	 * @throws CertificateException
+	 * @param proxyChain proxy chain
+	 * @param errors errors
+	 * @param unresolvedExtensions unresolved extensions
+	 * @param validDate valid date
+	 * @throws CertificateException certificate exception
 	 */
 	protected void checkProxyChainMain(X509Certificate[] proxyChain, 
 			List<ValidationError> errors, Set<String> unresolvedExtensions, Date validDate)
@@ -454,6 +469,10 @@ public class BCCertPathValidator
 	 * @param proxyCert certificate to be checked
 	 * @param errors out arg - list of errors found
 	 * @param position position in original chain to be used in error reporting
+	 * @param proxyChain proxy chain
+	 * @param validationTime validation time
+	 * @throws CertPathValidatorException certificate path validator exception
+	 * @throws CertificateParsingException certificate parsing exception
 	 */
 	protected void checkPairWithProxy(X509Certificate issuerCert, X509Certificate proxyCert, 
 			List<ValidationError> errors, int position, X509Certificate[] proxyChain, Date validationTime)
